"""Top-level package for pyutils.

A comprehensive Python utility library providing functions for array manipulation,
string processing, mathematical operations, object handling, function utilities,
asynchronous operations, and byte processing.

Ported from the jsutils JavaScript library to provide similar functionality in Python.
"""

__author__ = """mudssky"""
__email__ = "mudssky@gmail.com"
__version__ = "0.3.0"

# Import all modules
from . import (
    array,
    async_utils,
    collection,
    date,
    encoding,
    function,
    math,
    object,
    string,
    type_utils,
    url,
)
from . import bytes as bytes_utils

# Import commonly used functions for convenience
from .array import (
    alphabetical,
    chunk,
    diff,
    first,
    has_intersects,
    last,
    range_iter,
    range_list,
    shuffle,
    toggle,
    unique,
    zip_object,
)
from .async_utils import (
    delay,
    filter_async,
    map_async,
    race,
    retry_async,
    run_in_thread,
    sleep_async,
    timeout,
)
from .bytes import (
    Bytes,
    bytes_util,
    humanize_bytes,
    parse_bytes,
)
from .collection import (
    at,
    copy_within,
    entries,
    every,
    fill,
    find_index,
    find_last_index,
    flat_map,
    group_by,
    includes,
    keys,
    some,
    splice,
    to_reversed,
    to_sorted,
    values,
    with_item,
)
from .date import (
    add_days,
    add_hours,
    add_minutes,
    format_relative_time,
    get_time,
    is_valid_date,
    now,
    parse_date,
    to_date_string,
    to_iso_string,
    to_time_string,
)
from .encoding import (
    atob,
    btoa,
    decode_base64,
    decode_hex,
    encode_base64,
    encode_hex,
    escape_html,
    generate_random_string,
    hash_string,
    is_base64,
    is_hex,
    is_json,
    json_parse,
    json_stringify,
    unescape_html,
    url_decode,
    url_encode,
)
from .function import (
    debounce,
    memoize,
    once,
    throttle,
    with_retry,
)
from .math import (
    clamp,
    factorial,
    fibonacci,
    gcd,
    get_random_int,
    is_even,
    is_odd,
    is_prime,
    lcm,
    lerp,
    normalize,
)
from .object import (
    deep_copy,
    flatten_dict,
    get_nested_value,
    merge,
    omit,
    pick,
    safe_json_stringify,
    set_nested_value,
)
from .string import (
    camel_case,
    capitalize,
    dash_case,
    fuzzy_match,
    generate_uuid,
    parse_template,
    pascal_case,
    slugify,
    snake_case,
    trim,
    truncate,
)
from .type_utils import (
    is_array,
    is_boolean,
    is_date,
    is_empty,
    is_finite,
    is_function,
    is_integer,
    is_nan,
    is_null,
    is_number,
    is_object,
    is_regex,
    is_string,
    is_undefined,
    parse_float,
    parse_int,
    to_boolean,
    to_number,
    to_string,
    typeof,
)
from .url import (
    URLParser,
    build_url,
    decode_uri,
    decode_uri_component,
    encode_uri,
    encode_uri_component,
    get_domain,
    get_query_params,
    is_valid_url,
    parse_url,
)


# Define what gets exported when using "from pyutils import *"
__all__ = [
    "Bytes",
    "URLParser",
    "add_days",
    "add_hours",
    "add_minutes",
    "alphabetical",
    "array",
    "async_utils",
    "at",
    "atob",
    "btoa",
    "build_url",
    "bytes_util",
    "bytes_utils",
    "camel_case",
    "capitalize",
    "chunk",
    "clamp",
    "collection",
    "copy_within",
    "dash_case",
    "date",
    "debounce",
    "decode_base64",
    "decode_hex",
    "decode_uri",
    "decode_uri_component",
    "deep_copy",
    "delay",
    "diff",
    "encode_base64",
    "encode_hex",
    "encode_uri",
    "encode_uri_component",
    "encoding",
    "entries",
    "escape_html",
    "every",
    "factorial",
    "fibonacci",
    "fill",
    "filter_async",
    "find_index",
    "find_last_index",
    "first",
    "flat_map",
    "flatten_dict",
    "format_relative_time",
    "function",
    "fuzzy_match",
    "gcd",
    "generate_random_string",
    "generate_uuid",
    "get_domain",
    "get_nested_value",
    "get_query_params",
    "get_random_int",
    "get_time",
    "group_by",
    "has_intersects",
    "hash_string",
    "humanize_bytes",
    "includes",
    "is_array",
    "is_base64",
    "is_boolean",
    "is_date",
    "is_empty",
    "is_even",
    "is_finite",
    "is_function",
    "is_hex",
    "is_integer",
    "is_json",
    "is_nan",
    "is_null",
    "is_number",
    "is_object",
    "is_odd",
    "is_prime",
    "is_regex",
    "is_string",
    "is_undefined",
    "is_valid_date",
    "is_valid_url",
    "json_parse",
    "json_stringify",
    "keys",
    "last",
    "lcm",
    "lerp",
    "map_async",
    "math",
    "memoize",
    "merge",
    "normalize",
    "now",
    "object",
    "omit",
    "once",
    "parse_bytes",
    "parse_date",
    "parse_float",
    "parse_int",
    "parse_template",
    "parse_url",
    "pascal_case",
    "pick",
    "race",
    "range_iter",
    "range_list",
    "retry_async",
    "run_in_thread",
    "safe_json_stringify",
    "set_nested_value",
    "shuffle",
    "sleep_async",
    "slugify",
    "snake_case",
    "some",
    "splice",
    "string",
    "throttle",
    "timeout",
    "to_boolean",
    "to_date_string",
    "to_iso_string",
    "to_number",
    "to_reversed",
    "to_sorted",
    "to_string",
    "to_time_string",
    "toggle",
    "trim",
    "truncate",
    "type_utils",
    "typeof",
    "unescape_html",
    "unique",
    "url",
    "url_decode",
    "url_encode",
    "values",
    "with_item",
    "with_retry",
    "zip_object",
]
